;##################################################################
;
;   TesBwan for the TI-86 - low level graphics routines
;
;   Programmed by Patrick Davidson (pad@calc.org)
;        
;   This program is in the public domain.  There is no warranty.
;
;   This file was last updated August 22, 2002.
;
;##################################################################

;############## Interrupt routine
;
; Advances grayscale frame to the next one to be displayed.  Note that the
; next frame begins display one frame after the video address port is
; updated.
;
; After a frame has been show a sufficient number of times, the interrupt
; automatically switches to the other frame, and the rest of the program
; must keep up.  This is required due to the one-frame delay, since it is
; quite difficult to predict whether the main program will finish drawing
; one frame later.
;
; Since the cycle counter is reset to 0 after the display port is set, the
; new buffer address will only be displayed on the next interrupt, after
; which the cycle is set to 1.  Due to the one-frame delay, the next buffer
; only starts being shown when the cycle is 2.

interrupt_handler:
        push    af
        push    hl

        in      a,(3)               ; Exit if not VBL interrupt
        bit     1,a
        jr      z,iexit

        ld      hl,(gs_page)        ; H = F0 or F8 (depending on buffer)
                                    ; L = 0/2/4 (page counter)
        ld      a,h
        add     a,l                 ; A = sum of buffer / page
        and     $FC                 ; turns pseudo-page 2 to 0
        out     (0),a

        ld      a,2
        add     a,l                 ; A = next pseudo-page     
        cp      6                   ; if page counter reaches, then
        jr      nz,no_restart
        xor     a                   ; restart
no_restart:
        ld      (gs_page),a         ; store new page

        ld      hl,num_cycles       
        inc     (hl)                ; increment grayscale cycle number
        ld      a,6                 ; continue only if at end of cycle
        cp      (hl)
        jr      nz,iexit

        xor     a                   ; restart cycle
        ld      (hl),a
        ld      a,(gs_buffer)       ; alternate buffer used
        xor     8
        ld      (gs_buffer),a

        in      a,(3)                   ; Bit 1 = ON key status
iexit:  and     1
        add     a,9                     ; A = 10 if ON pressed, 9 if not
        out     (3),a                  
        ld      a,11
        out     (3),a                   
        pop     hl
        pop     af
        ei
        reti
interrupt_end:

;############## Initialization

init:   ld      hl,interrupt_handler
        ld      de,interrupt_entry
        ld      bc,interrupt_end-interrupt_handler
        ldir

        ld      hl,interrupt_table      ; Install interrupt
        ld      (hl),interrupt_byte
        ld      bc,256
        ld      de,interrupt_table+1
        ldir

        ld      a,interrupt_reg
        ld      i,a
        im      2

        ld      hl,$f000                ; set up grayscale data
        ld      (gs_page),hl
        xor     a
        ld      (num_cycles),a
        ld      hl,$f800
        ld      (gfx_buffer),hl
        ret

;############## Wait for next frame, and do page swap

wait_next_frame:
        ld      hl,num_cycles
        ld      a,1
lw1:    cp      (hl)
        jr      nz,lw1

        ld      a,2
lw2:    cp      (hl)
        jr      nz,lw2

        ld      a,(gs_buffer)
        xor     8
        ld      h,a
        ld      l,0
        ld      (gfx_buffer),hl
        ret

;############## Display a frame from the screen
;
; Displays the data buffer on the screen.

show_background:
        ld      hl,back_buffer+1
        ld      de,(gfx_buffer)
        ld      a,128
loop_copy_background:
        ldi
        ldi
        ldi
        ldi
        ldi
        ldi
        ldi
        ldi
        ldi
        ldi
        ldi
        ldi
        ldi
        ldi
        inc     hl
        inc     de
        inc     hl
        inc     de
        dec     a
        jr      nz,loop_copy_background
        ret

;############## Draw initial level data

draw_level:
        ld      de,back_buffer
        ld      hl,working_map
        ld      a,(screen_x)
        and     $f8
        ld      (screen_x),a
        rrca
        rrca
        rrca
        call    ADD_HL_A
        dec     hl
        ld      b,16
draw_level_loop:
        push    de
        push    hl
        push    bc
        call    draw_column
        pop     bc
        pop     hl
        pop     de
        inc     de
        inc     hl
        djnz    draw_level_loop
        ret

draw_column:
        ld      b,11
draw_column_loop:
        ld      c,6
        ld      a,1
        cp      b
        jr      nz,not_last
        ld      c,4
not_last:
        push    bc
        ld      a,(hl)
        ld      b,a
        add     a,a
        add     a,b
        add     a,a
        add     a,a                             ; A = brick number * 12
        push    hl
        ld      hl,tiles
        call    ADD_HL_A                        ; HL -> brick start

        push    de
        ld      b,c
loop_draw_brick_1:
        ld      a,(hl)
        ld      (de),a
        inc     hl
        ld      a,16
        call    ADD_DE_A
        djnz    loop_draw_brick_1

        ex      de,hl
        ex      (sp),hl
        ex      de,hl

        inc     d
        inc     d
        inc     d
        inc     d

        ld      b,c
        bit     1,b
        jr      nz,loop_draw_brick_2
        inc     hl
        inc     hl
loop_draw_brick_2:
        ld      a,(hl)
        ld      (de),a
        inc     hl
        ld      a,16
        call    ADD_DE_A
        djnz    loop_draw_brick_2
        pop     de

        pop     hl                              ; HL -> map
        ld      bc,MAP_WIDTH
        add     hl,bc                           ; HL -> next row in map
        pop     bc
        djnz    draw_column_loop
        ret

scroll_from_left:
        ld      a,(screen_x)
        or      a
        ret     z
        call    shift_screen_right
        ld      hl,screen_x
        dec     (hl)
        ld      a,(hl)
        and     7
        ret     nz
        ld      a,(hl)
        rra
        rra
        rra
        and     31
        ld      hl,working_map-1
        call    ADD_HL_A
        ld      de,back_buffer
        jp      draw_column

scroll_from_right:
        ld      a,(screen_x)
        cp      8*(MAP_WIDTH-14)
        ret     z
        call    shift_screen_left
        ld      hl,screen_x
        inc     (hl)
        ld      a,(hl)
        and     7
        ret     nz
        ld      a,(hl)
        rra
        rra
        rra
        and     31
        ld      hl,working_map+14
        call    ADD_HL_A
        ld      de,back_buffer+$F
        jp      draw_column

shift_screen_left:
        ld      hl,back_buffer+$7ff
        ld      b,0
lsl:    rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        rl      (hl)
        dec     hl
        djnz    lsl
        ret

shift_screen_right:
        ld      hl,back_buffer
        ld      b,0
lsr:    rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        rr      (hl)
        inc     hl
        djnz    lsr
        ret
